/*
  GPS avec affichage QRA Locator.
  GPS  entrée RX pin D0
  ---------------------
  Affichage sur shield LCD (2*16). Selection des données via le
  bouton poussoir "Select".
  Les Trames NMEA sont disponibles sur la sortie 1TX d'Arduino
  pour éventuellement moduler une platine Bluetooth.

  Module Bluetooth : Alimentation + 3 V impératif.
  Pin1 Digi arduino vers RX(2)de bluetooth.

  ATTENTION: Le téléchargement doit être réalisé sans le Shield GPS
  -----------------------------------------------------------------
  En foction de la configuration des librairies utilsez
  #include <eeprom.h> ou //#include <EEPROM.h>
  
  Version 2  fevrier 2014 (amélioration de la sélection)
  Version 3  juin 2015 (Décalage UTC/Heure locale via touches Up/Down)
                       (Jour + 1 selon décalage UTC/H locale)
                       
  Version31   Avril 2016 (bug affichage année)
  Version32   Mars  2017 (bug heure locale)
  Version33   Janvier  2019 (Modification des valeurs val_key)
  La valeur val_key foction de la platine utilisée s'affiche
  à la suite des trames lisibles sur le moniteur série.
  Appuyer sur chacune des touches afin de voir la valeur correspondante
  qu'il faudra éventuellement modifier dans le programme
  
   
  F5MNA    Site http://f5mna.free.fr
 */

//Concerne GPS
#include <TinyGPS.h>
TinyGPS gps;
#include <floatToString.h>
//#include <EEPROM.h>
#include <eeprom.h>
//LCD
#include <LiquidCrystal.h>
// LCD RS pin to digital pin 8
// LCD Enable pin to digital pin 9
// LCD D4 pin to digital pin 4
// LCD D5 pin to digital pin 5
// LCD D6 pin to digital pin 6
// LCD D7 pin to digital pin 7
// LCD R/W pin to ground
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

//Boutons
#define Touche_PIN  A0    // Entrée des infos boutons
int Val, n, h, Pos, hour0, FdM;
int sensorPin = A0;
int sensorValue = 0;
int val_key;
boolean QRX = false;

static void gpsdump(TinyGPS &gps);
static bool feedgps();
static void print_float(float val, float invalid, int len, int prec);
static void print_int(unsigned long val, unsigned long invalid, int len);
static void print_date(TinyGPS &gps);
float latitude, longitude, altitude, speed_kmph;

//////////////////////////////////////////////////////////////
void setup()
{
  Serial.begin (9600);
  pinMode( Touche_PIN, INPUT );       //
  digitalWrite( Touche_PIN, LOW );    //
  n = 0;
  h = EEPROM.read (0);

  //Presentation
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print ("      GPS       ");
  lcd.setCursor(0, 1);
  lcd.print ("F5MNA Version 31");
  delay(2000);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("  En cours...   ");
  lcd.setCursor(0, 1);
  lcd.print(" Patientez SVP  ");
  Serial.println ("Recherche trames");
  Serial.println ("  Patientez     ");
}
/////////////////////////////////////////////////////////////
void GPS()
////////////////////////////////////////////////////////////
{
  bool newData = false;
  unsigned long start = millis();
  unsigned long chars;
  unsigned short sentences, failed;

  // Analyse des trames pendant 1 seconde
  for (unsigned long start = millis(); millis() - start < 1000;)
  {
    while (Serial.available())
    {
      char c = Serial.read();
      Serial.write(c);   //Ne pas concerver pour ne pas afficher les trames 
      if (gps.encode(c)) //Si une nouvelle trame correcte arrive
        newData = true;
    }
  }
  gps.stats(&chars, &sentences, &failed);
  Serial.print(chars);
  gpsdump(gps);
}

static void gpsdump(TinyGPS &gps)
{
  unsigned long age, date, time, chars = 0;
  unsigned short sentences = 0, failed = 0;
  print_date(gps);
  //Serial.println();
  gps.f_get_position(&latitude, &longitude);

  /////////////////////////////////
  //// LATITUDE & LONGITUDE   /////
  /////////////////////////////////
  if (n == 1) {
    lcd.setCursor( 0, 0 );
    lcd.print(" Lat: ");
    lcd.print(latitude, 5);
    lcd.print("  ");

    lcd.setCursor( 0, 1 );
    lcd.print(" Long: ");
    lcd.print(longitude, 5);
    lcd.print("  ");
  }
  /////////////////////////////
  /// VITESSE / DIRECTION /////
  /////////////////////////////
  if (n == 3) {
    lcd.setCursor( 0, 0 );
    lcd.print("    ");
    lcd.print(int(gps.f_speed_kmph()));
    lcd.print(" Km/h  ");

    lcd.setCursor( 0, 1 );
    lcd.print(" Dir: ");
    lcd.print(int(gps.f_course()));
    lcd.print(" Deg");
  }
}
////////////////////////
////  DATE & HEURE  ////
////////////////////////
static void print_date(TinyGPS &gps)
{
  int year;
  byte month, day, hour, minute, second, hundredths;
  unsigned long age;
  gps.crack_datetime(&year, &month, &day, &hour, &minute, &second, &hundredths, &age);
  {
    char sz[32];
    sprintf(sz, "  %02d/%02d/%02d %02d:%02d:%02d   ",
            day, month, year, hour , minute, second); /// + h décalage / UTC

    /////////// Passage à J+1 lorsque H décalée/UTC = 0 //////////
    ////////////// En fin de mois n passe a mois+1 ///////////////
      if ((hour + h) > 23) day = day + 1;
      if ((h < 0) and (hour + h) > 0) day = day - 1; 
       
      if ((FdM == 0) and (day == 31)) {
      day = 1;
      month = (month + 1);  
      }

      if ((FdM == 1) and (day == 32)) {
      day = 1;
      month = (month + 1);
      if (month == 13)
      {month = 1;
      year = year + 1;   
      }}
      
      if ((FdM == 2) and (day == 29)) {
      day = 1;
      month = (month + 1);    
      }} 
   
    //////////////////////////////////////////////////////////////
    if (n == 0) {
      lcd.setCursor (0, 0 );
      switch (Pos) {
        case 1:
          lcd.print(" ");
          break;

        case  2:
          lcd.print("  ");
          break;

        case 3:
          lcd.print("   ");
          break;
      }

      if ((day < 10) and (QRX == true))  lcd.print(" ");
      if (QRX == true) lcd.print(day);
      switch (month) {
        case 1:
          if (QRX == true) lcd.print  (" janvier ");
          Pos = 1;  //Décalage affichage
          FdM = 1;  // Jour fin de mois
          break;
        case 2:
          if (QRX == true) lcd.print  (" fevrier ");
          Pos = 1;
          FdM = 2;
          break;
        case 3:
          if (QRX == true) lcd.print (" mars ");
          Pos = 2;
          FdM = 1;
          break;
        case 4:
          if (QRX == true) lcd.print (" avril ");
          Pos = 2;
          FdM = 0;
          
          break;
        case 5:
          if (QRX == true) lcd.print (" mai ");
          Pos = 3;
          FdM = 1;
          break;
        case 6:
          if (QRX == true) lcd.print (" juin ");
          Pos = 2;
          FdM = 0;
          break;
        case 7:
          if (QRX == true) lcd.print (" juillet ");
          Pos = 1;
          FdM = 1;
          break;
        case 8:
          if (QRX == true) lcd.print (" aout ");
          Pos = 2;
          FdM = 1;
          break;
        case 9:
          if (QRX == true) lcd.print (" septemb.");
          Pos = 1;
          FdM = 0;
          break;
        case 10:
          if (QRX == true) lcd.print (" octobre ");
          Pos = 1;
          FdM = 1;
          break;
        case 11:
          if (QRX == true) lcd.print (" novembre ");
          Pos = 0;
          FdM = 0;
          break;
        case 12:
          if (QRX == true) lcd.print (" decembre ");
          Pos = 0;
          FdM = 1;
          break;
      }

      if (QRX == true)
      {lcd.print (year);
      lcd.print (" ");}
      Serial.println (year);      
      Serial.print (hour);
      lcd.print(": "); 
      Serial.print (minute);
      lcd.print(": ");
      Serial.println(second);
      
      
      if (QRX == true) lcd.print ("      ");
      QRX = true;
      hour = (hour + h);
      lcd.setCursor(0, 1 );
      lcd.print("    ");
      if (hour > 23) (hour = 0);
      if (hour < 10) (lcd.print ("0"));
      lcd.print(hour);
      lcd.print(":");
      if (minute < 10) (lcd.print ("0"));
      lcd.print(minute);
      lcd.print(":");
      if (second < 10) (lcd.print ("0"));
      lcd.print(second);
      lcd.print("   ");
     }
  feedgps();
}
////////////////////////////////////

static bool feedgps()
{
  while (Serial.available())
  {
    if (gps.encode(Serial.read()))
      return true;
  }
  return false;
}
////////////////////////
void Locator ()
//////////////////////
{
  float symb1, symb2, symb3, symb4, symb5, symb6; //,latitude, longitude;
  //latitude = 46.71446;
  //longitude = -5.64391;
  symb1 = ((180 + longitude) / 20);
  symb2 = ((90 + latitude) / 10);
  symb3 = (symb1 - int(symb1)) * 10;
  symb4 = (symb2 - int(symb2)) * 10;
  symb5 = (symb3 - int(symb3)) * 24;
  symb6 = (symb4 - int(symb4)) * 24;

  if (n == 2)
  {
    lcd.setCursor( 0, 0 );
    lcd.print ("Locator: ");
    lcd.print (char(int(symb1) + 65));
    lcd.print (char(int(symb2) + 65));
    lcd.print (int(symb3));
    lcd.print (int(symb4));
    lcd.print (char(int(symb5) + 65));
    lcd.print (char(int(symb6) + 65));
    lcd.print (" ");
    lcd.setCursor( 0, 1 );
    lcd.print ("Altitude: ");
    lcd.print (int (gps.f_altitude()));
    lcd.print (" m  ");
  }
}



void loop ()
{
  while (Serial.available())    // Tant que des données sont en entrée..
  {
    int c = Serial.read();      // Charge les données trames...
    if (gps.encode(c))          // Si la trame NMEA est correcte...
    {
      sensorValue = analogRead(sensorPin); // lecture de la touche
      val_key = int(sensorValue / 100);     // valeur entière de la touche / 10
      Serial.print ("val_key: ");
      Serial.println (val_key);
      switch (val_key)
      {
        // Avance heure / UTC  (Up/Down)
          case 0 :
          Serial.print ("TEST ");
          lcd.setCursor ( 0, 0 );
          lcd.print ("  H locale/UTC   ");
          lcd.setCursor ( 0, 1 );
          h = h + 1;
          if (h > 0)  lcd.print("      +");
          if (h == 0) lcd.print("       ");
          if (h < 0)  lcd.print("      ");
          lcd.print (h);
          lcd.print(" H      ");
          EEPROM.write (0,h);
          if (h == 13)  (h = 0);
          delay (150);
          break;

        case 2:
          lcd.setCursor ( 0, 0 );
          lcd.print ("  H locale/UTC   ");
          lcd.setCursor ( 0, 1 );
          h = h - 1;
          if (h > 0)  lcd.print("      +");
          if (h == 0) lcd.print("       ");
          if (h < 0)  lcd.print("      ");
          lcd.print (h);
          lcd.print(" H      ");
          if (h == -13) (h = 0);
           EEPROM.write (0,h);
          delay (150);
          break;


        case 6 : //Select
          n = n  + 1;
          if (n >= 4) (n = 0);
          lcd.setCursor ( 0, 0 );
          lcd.print ("   Selection    ");
          lcd.setCursor( 0, 1 );
          if (n == 0) (lcd.print ("  Date / Heure  "));
          if (n == 1) (lcd.print ("  Coordonnees   "));
          if (n == 2) (lcd.print (" Locator / Alti "));
          if (n == 3) (lcd.print (" Vitesse / dir. "));
          delay (250);
          break;

        case 10 :
          GPS();
          Locator();
          break;
      }
    }
  }
}



